"""全アダプターをインポートしてレジストリに登録する。"""

import gfo.adapter.azure_devops  # noqa: F401
import gfo.adapter.backlog  # noqa: F401
import gfo.adapter.bitbucket  # noqa: F401
import gfo.adapter.forgejo  # noqa: F401
import gfo.adapter.gitbucket  # noqa: F401
import gfo.adapter.gitea  # noqa: F401
import gfo.adapter.github  # noqa: F401
import gfo.adapter.gitlab  # noqa: F401
import gfo.adapter.gogs  # noqa: F401
